##
## Generic makefile that copies settings from the corresponding VC++ .vcxproj file
## Win32-Emscripten      
##


##
## These variables can be overriden using environment variables
##

EmscriptenPath         :=C:/emsdk/emscripten/1.38.26/
AR       	       := $(EmscriptenPath)em++ rcus
CXX      	       := $(EmscriptenPath)em++
CC       	       := $(EmscriptenPath)emcc
LinkerName             :=$(EmscriptenPath)em++
SharedObjectLinkerName :=$(EmscriptenPath)em++ -shared -fPIC
IncludeSwitch          :=-I
LibrarySwitch          :=-l
LibraryPathSwitch      :=-L
MakeDirCommand         :=makedir
IncludePath            := $(InclPaths:%=$(IncludeSwitch)%) 
Libs                   := $(libs:%=$(LibrarySwitch)%)
LibPath                := $(LibPaths:%=$(LibraryPathSwitch)%) 



##
## Helper Functions to extract .c/.cpp/.h files from corresponding .vcxproj-file
##

define exclNode 	# prints xml-tag ExcludedFromBuild (= true/false - if exists)
$(shell tools\ssed  "s/xmlns=\"\(.*\)\"//" $(ProjectFileName) | tools\xml sel -T -t -v "/Project/ItemGroup/$(2)[@Include=\"$(1)\"]/ExcludedFromBuild")
endef

define checkFile	# e.g., $(1)=file.c, $(2)=ClCompile. Returns $(1) if $(2)'s xml-tag does not include ExcludedFromBuild == true
$(if $(filter $(call exclNode,$(1),$(2)),true),,$(1))
endef

define extract 
$(foreach file,$(shell tools\ssed  "s/xmlns=\"\(.*\)\"//"  $(ProjectFileName) | tools\xml sel -T -t -v "/Project/ItemGroup/$(1)/@Include"),$(call checkFile,$(file),$(1)))
endef

src_files	:=$(call extract,ClCompile)
h_files		:=$(call extract,ClInclude)
c_files		:=$(filter %.c,$(src_files))
cxx_files	:=$(filter-out %.c,$(src_files))
o_files		:=$(foreach l,$(c_files),$(IntDir)$(notdir $(l)).o) $(foreach l,$(cxx_files),$(IntDir)$(notdir $(l)).o)


# Gcc/Clang will create these .d files containing dependencies.
dependencies = $(o_files:%.o=%.d)


##
## Main Build Targets 
##
.PHONY: all clean
all: $(IntDir)index.js

# Rule to compile index.js from all .o files
$(IntDir)index.js: $(o_files) 
	@echo Linking...
#	@$(MakeDirCommand) $$(@D)
	$(LinkerName) $^ $(CXXFLAGS) $(EMFLAGS) -o $(IntDir)index.js $(LibPath) $(Libs)


##
## Objects
##
## Create build rule for every single object file.
## 'foreach' below creates a build-target rule per source file
## The potential dependency on header files is covered by calling `-include $(DEP)` below
## The -MMD flags below additionaly creates a .d file with the same name as the .o file.

# Include all .d files
-include $(dependencies)


define CPP_RULES
$(IntDir)$(notdir $(1)).o: % : $(1)
	$(CXX) $(CXXFLAGS) $(EMFLAGS) $(IncludePath) -MMD $$< -o $$@
endef
$(foreach l,$(cxx_files),$(eval $(call CPP_RULES,$(l))))

define C_RULES
$(IntDir)$(notdir $(1)).o: % : $(1)
	$(CC) $(CFLAGS) $(EMFLAGS) $(IncludePath) -MMD $$< -o $$@
endef
$(foreach l,$(c_files),$(eval $(call C_RULES,$(l))))


##
## Clean - unused -
##

ifeq ($(OS),Windows_NT)
    RM = cmd //C del //Q //F
    RRM = cmd //C rmdir //Q //S
else
    RM = rm -f
    RRM = rm -f -r
endif

clean:
	$(RM) -r ./$(IntDir)


